// SettingsDb.cpp: implementation of the SettingsDb class.
//
//////////////////////////////////////////////////////////////////////

#include "SettingsDb.h"

namespace sq3 {
    
	/**
	   Internal implementation details for SettingsDb
	*/
	class SettingsDbImpl
	{
	public:
		CString filename;
		sq3::Database * db;
        
		SettingsDbImpl( CString const & fn );
		~SettingsDbImpl();
	private:
		void InitDb();
	};

   
	void SettingsDbImpl::InitDb()
	{
		if( ! this->db->IsOpen() ) 
		{
			return;
		}
		this->db->Execute( "create table if not exists settings(key PRIMARY KEY ON CONFLICT REPLACE,value)" );
		//this->db->Execute( "PRAGMA temp_store_directory = 'C:\\'" ); // C:\\ is certainly much faster than my SD card!
		// ^^^ Per the sqlite3 docs, temp files are actually unlinked immediately after they are opened, so they
		// aren't visible in the filesystem.
		// Doesn't appear to make much of a speed diff under CE:
		this->db->Execute( "PRAGMA temp_store = MEMORY" ); // i don't like this, but want to speed up access
		this->db->Execute( "PRAGMA synchronous = OFF" ); // again: i don't like this but want more speed
	}
    
    
	SettingsDbImpl::SettingsDbImpl( CString const & fn )
		: filename(fn), db( new sq3::Database(fn) )
	{
		this->InitDb();
	}
	SettingsDbImpl::~SettingsDbImpl()
	{
		delete this->db;
	}
    
	SettingsDb::SettingsDb( CString const & filename )
		: m_pimpl( new SettingsDbImpl( filename ) )
	{
        
	}
    
	SettingsDb::~SettingsDb()
	{
		delete this->m_pimpl;
	}

	Database & SettingsDb::GetDb()
	{
		return *(this->m_pimpl->db);
	}

	bool SettingsDb::IsOpen() const
	{
		return this->m_pimpl->db->IsOpen();
	}

	int SettingsDb::Clear()
	{
		return this->GetDb().Execute( "delete from settings" );
	}

	int SettingsDb::Clear( CString const & where )
	{
		CString del;
		del.Format( _T("delete from settings %s"), where );
		return this->GetDb().Execute( del );
	}

	/** Internal shortcut to save duplicate code in the Set() overloads. */
	static CString SettingsDb_Set_SQL = _T("insert into settings values(?,?)" );
   
	void SettingsDb::Set( CString const & key, int val )
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Set_SQL );
		st.Bind( 1, key );
		st.Bind( 2, val );
		st.ExecuteNonQuery();
	}

	void SettingsDb::Set( CString const & key, int64_t val )
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Set_SQL );
		st.Bind( 1, key );
		st.Bind( 2, val );
		st.ExecuteNonQuery();
	}
    
	void SettingsDb::Set( CString const & key, bool val )
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Set_SQL );
		st.Bind( 1, key );
		st.Bind( 2, val ? (int) 1 : (int) 0);
		st.ExecuteNonQuery();
	}
    

	void SettingsDb::Set( CString const & key, double val )
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Set_SQL );
		st.Bind( 1, key );
		st.Bind( 2, val );
		st.ExecuteNonQuery();
	}
    
	void SettingsDb::Set( CString const & key, CString const & val )
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Set_SQL );
		st.Bind( 1, key );
		st.Bind( 2, val );
		st.ExecuteNonQuery();
	}

	void SettingsDb::Set( CString const & key, LPCTSTR val )
	{
		if( ! val ) return;
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Set_SQL );
		st.Bind( 1, key );
		int len;
		for( len = 0; val[len] != 0; ++len ) {}
		CString v( val, len );
		st.Bind( 2, v );
		st.ExecuteNonQuery();
	}

	void SettingsDb::Unset( CString const & key )
	{
		sq3::Statement st( *(this->m_pimpl->db), _T("delete from settings where key like ?" ) );
		st.Bind( 1, key );
		st.ExecuteNonQuery();
	}

	/** Internal shortcut to save duplicate code in the Get() overloads. */
	static CString SettingsDb_Get_SQL = _T("select value from settings where key like ?");

	bool SettingsDb::Get( CString const & key, int & val ) const
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Get_SQL );
		st.Bind( 1, key );
		return SQLITE_OK == st.ExecuteInt( val );
	}

	bool SettingsDb::Get( CString const & key, int64_t & val ) const
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Get_SQL );
		st.Bind( 1, key );
		return SQLITE_OK == st.ExecuteInt64( val );
	}

	bool SettingsDb::Get( CString const & key, bool & val ) const
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Get_SQL );
		st.Bind( 1, key );
		int tval = val ? 1 : 0;
		if( SQLITE_OK == st.ExecuteInt( tval ) )
		{
			val = (0 != tval);
			return true;
		}
		return false;
	}
    

	bool SettingsDb::Get( CString const & key, double & val ) const
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Get_SQL );
		st.Bind( 1, key );
		return SQLITE_OK == st.ExecuteDouble( val );
	}
    
	bool SettingsDb::Get( CString const & key, CString & val ) const
	{
		sq3::Statement st( *(this->m_pimpl->db), SettingsDb_Get_SQL );
		st.Bind( 1, key );
		return SQLITE_OK == st.ExecuteString( val );
	}
    
} // namespace
