// SettingsDb.h: interface for the SettingsDb class.
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SETTINGSDB_H__CD5608D6_DBF8_4942_8B6D_5ABBA16E8657__INCLUDED_)
#define AFX_SETTINGSDB_H__CD5608D6_DBF8_4942_8B6D_5ABBA16E8657__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afx.h> // CString

#include "SQ3.h"

namespace sq3 {

	class SettingsDbImpl; // for internal use only

	/**
	   SettingsDb is an abstraction around a database
	   of key/value pairs.

	   General notes:

	   - The underlying database (passed to the ctor) may not be
	   opened by another object or process. Doing so will cause
	   the db to not open and get/set operations will fail.

	   - Each SettingsDb object maps to one underlying physical
	   database with a single table named 'settings' and two
	   fields named 'key' and 'value'. These fields can hold integer,
	   double, or unicode string values.

	   - Keys are unique within a given database and are treated
	   case-insensitively. Keys may not have apostrophes or
	   double-quote characters in them. It is safest to use only
	   alphanumeric and underscore characters, though spaces are
	   also allowed.

	   - Results may be undefined if keys contain non-latin1 characters
	   (e.g., umlauts). This "should work" but is untested.

	*/
	class SettingsDb
	{
	public:
		/**
		   Opens or creates the given db file. Note that it may not
		   be legal for multiple instances to have the same database
		   open at once. Use IsOpen() to determine whether the db
		   was successfully opened.
		*/
		explicit SettingsDb( CString const & filename );
		/** Closes the db and releases internal resources. */
		~SettingsDb();

		/**
		   Sets the given key to the given value.
		   Setting an existing key replaces its value with val.
		*/
		void Set( CString const & key, int val );
		/**
		   Sets the given key to the given value.
		   Setting an existing key replaces its value with val.
		*/
		void Set( CString const & key, double val );
		/**
		   Sets the given key to the given value.
		   Setting an existing key replaces its value with val.
		*/
		void Set( CString const & key, CString const & val );

		/**
		   Sets the given key to the given value.
		   Setting an existing key replaces its value with val.
		*/
		void Set( CString const & key, int64_t );
		/**
		   This overload exists because if it doesn't then
		   passing a LPCTSTR to Set() will cause the bool overload
		   to be picked up by the compiler! Bug or feature? You decide.

		   If val is null then this function does nothing. val MUST
		   be null-terminated or undefined behaviour will result.
		*/
		void Set( CString const & key, LPCTSTR val );

		/**
		   Sets key to the given value. The bool will be saved as a 0 or 1
		   in the database.
		*/
		void Set( CString const & key, bool val );

		/**
		   Removes the given key from the db.
		*/
		void Unset( CString const & key );

		/**
		   If the db is opened and key is in the db, val is set to its value and true
		   is returned, else val is unchanged and false is returned.
		*/
		bool Get( CString const & key, int & val ) const;
		/**
		   If the db is opened and key is in the db, val is set to its value and true
		   is returned, else val is unchanged and false is returned.
		*/
		bool Get( CString const & key, double & val ) const;
		/**
		   If the db is opened and key is in the db, val is set to its value and true
		   is returned, else val is unchanged and false is returned.
		*/
		bool Get( CString const & key, CString & val ) const;

		/**
		   If the db is opened and key is in the db, val is set to its value and true
		   is returned, else val is unchanged and false is returned.
		*/
		bool Get( CString const & key, int64_t & val) const;

		/**
		   If the db is opened and key is in the db, val is set to its value and true
		   is returned, else val is unchanged and false is returned.

		   For purposes of converting to a boolean, any INTEGER value of non-zero
		   is considered to be true, and 0 is considered to be false.
		*/
		bool Get( CString const & key, bool & val ) const;

		/**
		   Returns true if this db was successfully opened.
		*/
		bool IsOpen() const;

		/**
		   Returns the underlying Database object, in case you want to play
		   directly with it.
		*/
		Database & GetDb();

		/**
		   Removes ALL entries in the settings table.

		   Return value is that of the underlying sqlite3_exec() call.
		*/
		int Clear();
		/**
		   Removes all entries in the settings table which match the given
		   where clause. The clause should be in the following form:

		   "where ..."

		   that is, it should start with the word 'where'.

		   Return value is that of the underlying sqlite3_exec() call.
		*/
		int Clear( CString const & where );

	private:
		mutable SettingsDbImpl * m_pimpl;
	};

} // namespace
#endif // !defined(AFX_SETTINGSDB_H__CD5608D6_DBF8_4942_8B6D_5ABBA16E8657__INCLUDED_)
